@@ -1,12 +1,14 @@
 import os
 import StringIO
+import threading
 
 import py.test
 
 from doit.dependency import Dependency
 from doit.task import Task
-from doit.cmds import doit_list, doit_run, doit_clean, doit_forget, doit_ignore
 from doit.main import InvalidCommand
+from doit import cmds
+
 
 
 TESTDB = os.path.join(os.path.dirname(__file__), "testdb")
@@ -24,62 +26,79 @@ TASKS_SAMPLE = [Task("t1", [""], doc="t1 doc string"),
                 Task("t3", [""], doc="t3 doc string")]
 TASKS_SAMPLE[2].task_dep = ['g1.a', 'g1.b']
 
+
 class TestCmdList(object):
 
-    def testListTasksWithDoc(self):
+    def testDefault(self):
         output = StringIO.StringIO()
-        doit_list(TESTDB, TASKS_SAMPLE, output, [], False, False, False)
+        cmds.doit_list(TESTDB, TASKS_SAMPLE, output, [])
+        got = [line for line in output.getvalue().split('\n') if line]
+        expected = [t.name for t in TASKS_SAMPLE if not t.is_subtask]
+        assert expected == got
+
+    def testDoc(self):
+        output = StringIO.StringIO()
+        cmds.doit_list(TESTDB, TASKS_SAMPLE, output, [], print_doc=True)
         got = [line for line in output.getvalue().split('\n') if line]
         expected = []
         for t in TASKS_SAMPLE:
             if not t.is_subtask:
-                expected.append("%s : %s" % (t.name, t.doc))
-        assert expected == got, output.getvalue()
-
-    def testListTasksWithDocQuiet(self):
-        output = StringIO.StringIO()
-        doit_list(TESTDB, TASKS_SAMPLE, output, [], False, True, False)
-        got = [line for line in output.getvalue().split('\n') if line]
-        expected = [t.name for t in TASKS_SAMPLE if not t.is_subtask]
-        assert expected == got, output.getvalue()
-
-    def testListAllTasksWithDoc(self):
-        output = StringIO.StringIO()
-        doit_list(TESTDB, TASKS_SAMPLE, output, [], True, False, False)
-        got = [line for line in output.getvalue().split('\n') if line]
-        expected = ["%s : %s" % (t.name, t.doc) for t in TASKS_SAMPLE]
+                expected.append("%s\t* %s" % (t.name, t.doc))
         assert expected == got
 
-    def testListAllTasksWithDocQuiet(self):
+    def testSubTask(self):
         output = StringIO.StringIO()
-        doit_list(TESTDB, TASKS_SAMPLE, output, [], True, True, False)
+        cmds.doit_list(TESTDB, TASKS_SAMPLE, output, [], print_subtasks=True)
         got = [line for line in output.getvalue().split('\n') if line]
         expected = [t.name for t in TASKS_SAMPLE]
         assert expected == got
 
-    def testListFilter(self):
+    def testFilter(self):
         output = StringIO.StringIO()
-        doit_list(TESTDB, TASKS_SAMPLE, output, ['g1', 't2'],
-                  False, True, False)
+        cmds.doit_list(TESTDB, TASKS_SAMPLE, output, ['g1', 't2'])
         got = [line for line in output.getvalue().split('\n') if line]
         expected = ['g1', 't2']
         assert expected == got
 
-    def testListFilterAll(self):
+    def testFilterSubtask(self):
         output = StringIO.StringIO()
-        doit_list(TESTDB, TASKS_SAMPLE, output, ['g1'], True, True, False)
+        cmds.doit_list(TESTDB, TASKS_SAMPLE, output, ['g1.a'])
+        got = [line for line in output.getvalue().split('\n') if line]
+        expected = ['g1.a']
+        assert expected == got
+
+    def testFilterAll(self):
+        output = StringIO.StringIO()
+        cmds.doit_list(TESTDB, TASKS_SAMPLE, output, ['g1'], print_subtasks=True)
         got = [line for line in output.getvalue().split('\n') if line]
         expected = ['g1', 'g1.a', 'g1.b']
         assert expected == got
 
     def testStatus(self):
         output = StringIO.StringIO()
-        doit_list(TESTDB, TASKS_SAMPLE, output, ['g1'],
-                  False, True, True)
+        cmds.doit_list(TESTDB, TASKS_SAMPLE, output, ['g1'], print_status=True)
         got = [line for line in output.getvalue().split('\n') if line]
         expected = ['R g1']
         assert expected == got
 
+    def testNoPrivate(self):
+        task_list = list(TASKS_SAMPLE)
+        task_list.append(Task("_s3", [""]))
+        output = StringIO.StringIO()
+        cmds.doit_list(TESTDB, task_list, output, ['_s3'])
+        got = [line for line in output.getvalue().split('\n') if line]
+        expected = []
+        assert expected == got
+
+    def testWithPrivate(self):
+        task_list = list(TASKS_SAMPLE)
+        task_list.append(Task("_s3", [""]))
+        output = StringIO.StringIO()
+        cmds.doit_list(TESTDB, task_list, output, ['_s3'], print_private=True)
+        got = [line for line in output.getvalue().split('\n') if line]
+        expected = ['_s3']
+        assert expected == got
+
 
 class TestCmdForget(object):
 
@@ -105,7 +124,7 @@ class TestCmdForget(object):
 
     def testForgetAll(self, tasks):
         output = StringIO.StringIO()
-        doit_forget(TESTDB, tasks, output, [])
+        cmds.doit_forget(TESTDB, tasks, output, [])
         got = output.getvalue().split("\n")[:-1]
         assert ["forgeting all tasks"] == got, repr(output.getvalue())
         dep = Dependency(TESTDB)
@@ -114,7 +133,7 @@ class TestCmdForget(object):
 
     def testForgetOne(self, tasks):
         output = StringIO.StringIO()
-        doit_forget(TESTDB, tasks, output, ["t2", "t1"])
+        cmds.doit_forget(TESTDB, tasks, output, ["t2", "t1"])
         got = output.getvalue().split("\n")[:-1]
         assert ["forgeting t2", "forgeting t1"] == got
         dep = Dependency(TESTDB)
@@ -123,7 +142,7 @@ class TestCmdForget(object):
 
     def testForgetGroup(self, tasks):
         output = StringIO.StringIO()
-        doit_forget(TESTDB, tasks, output, ["g2"])
+        cmds.doit_forget(TESTDB, tasks, output, ["g2"])
         got = output.getvalue().split("\n")[:-1]
 
         dep = Dependency(TESTDB)
@@ -137,14 +156,14 @@ class TestCmdForget(object):
     # if task dependency not from a group dont forget it
     def testDontForgetTaskDependency(self, tasks):
         output = StringIO.StringIO()
-        doit_forget(TESTDB, tasks, output, ["t3"])
+        cmds.doit_forget(TESTDB, tasks, output, ["t3"])
         dep = Dependency(TESTDB)
         assert None == dep._get("t3", "dep")
         assert "1" == dep._get("t1", "dep")
 
     def testForgetInvalid(self, tasks):
         output = StringIO.StringIO()
-        py.test.raises(InvalidCommand, doit_forget,
+        py.test.raises(InvalidCommand, cmds.doit_forget,
                        TESTDB, tasks, output, ["XXX"])
 
 
@@ -153,21 +172,21 @@ class TestCmdRun(object):
     def testProcessRun(self):
         remove_testdb()
         output = StringIO.StringIO()
-        doit_run(TESTDB, TASKS_SAMPLE, output)
+        cmds.doit_run(TESTDB, TASKS_SAMPLE, output)
         got = output.getvalue().split("\n")[:-1]
         assert [".  t1", ".  t2", ".  g1.a", ".  g1.b", ".  t3"] == got
 
     def testProcessRunFilter(self):
         remove_testdb()
         output = StringIO.StringIO()
-        doit_run(TESTDB, TASKS_SAMPLE, output, ["g1.a"])
+        cmds.doit_run(TESTDB, TASKS_SAMPLE, output, ["g1.a"])
         got = output.getvalue().split("\n")[:-1]
         assert [".  g1.a"] == got, repr(got)
 
     def testInvalidReporter(self):
         remove_testdb()
         output = StringIO.StringIO()
-        py.test.raises(InvalidCommand, doit_run,
+        py.test.raises(InvalidCommand, cmds.doit_run,
                 TESTDB, TASKS_SAMPLE, output, reporter="i dont exist")
 
     def testSetVerbosity(self):
@@ -178,13 +197,13 @@ class TestCmdRun(object):
         def my_execute(out, err, verbosity):
             used_verbosity.append(verbosity)
         t.execute = my_execute
-        doit_run(TESTDB, [t], output, verbosity=2)
+        cmds.doit_run(TESTDB, [t], output, verbosity=2)
         assert 2 == used_verbosity[0], used_verbosity
 
 
     def test_outfile(self):
         remove_testdb()
-        doit_run(TESTDB, TASKS_SAMPLE, 'test.out', ["g1.a"])
+        cmds.doit_run(TESTDB, TASKS_SAMPLE, 'test.out', ["g1.a"])
         try:
             outfile = open('test.out', 'r')
             got = outfile.read()
@@ -199,25 +218,35 @@ class TestCmdClean(object):
 
     def pytest_funcarg__tasks(self, request):
         def create_tasks():
-            self.count = 0
-            self.tasks = [Task("t1", None, clean=[(self.increment,)]),
-                          Task("t2", None, clean=[(self.increment,)]),]
+            self.cleaned = []
+            def myclean(name):
+                self.cleaned.append(name)
+            self.tasks = [Task("t1", None, dependencies=[':t2'],
+                               clean=[(myclean,('t1',))]),
+                          Task("t2", None, clean=[(myclean,('t2',))]),]
         return request.cached_setup(
             setup=create_tasks,
             scope="function")
 
-    def increment(self):
-        self.count += 1
-        return True
-
     def test_clean_all(self, tasks):
         output = StringIO.StringIO()
-        doit_clean(self.tasks, output, False, [])
-        assert 2 == self.count
+        cmds.doit_clean(self.tasks, output, False, False, [])
+        assert ['t1','t2'] == self.cleaned
 
     def test_clean_selected(self, tasks):
         output = StringIO.StringIO()
-        doit_clean(self.tasks, output, False, ['t2'])
+        cmds.doit_clean(self.tasks, output, False, False, ['t2'])
+        assert ['t2'] == self.cleaned
+
+    def test_clean_taskdep(self, tasks):
+        output = StringIO.StringIO()
+        cmds.doit_clean(self.tasks, output, False, True, ['t1'])
+        assert ['t2', 't1'] == self.cleaned
+
+    def test_clean_taskdep_once(self, tasks):
+        output = StringIO.StringIO()
+        cmds.doit_clean(self.tasks, output, False, True, ['t1', 't2'])
+        assert ['t2', 't1'] == self.cleaned
 
 
 class TestCmdIgnore(object):
@@ -242,7 +271,7 @@ class TestCmdIgnore(object):
 
     def testIgnoreAll(self, tasks):
         output = StringIO.StringIO()
-        doit_ignore(TESTDB, tasks, output, [])
+        cmds.doit_ignore(TESTDB, tasks, output, [])
         got = output.getvalue().split("\n")[:-1]
         assert ["You cant ignore all tasks! Please select a task."] == got, got
         dep = Dependency(TESTDB)
@@ -251,7 +280,7 @@ class TestCmdIgnore(object):
 
     def testIgnoreOne(self, tasks):
         output = StringIO.StringIO()
-        doit_ignore(TESTDB, tasks, output, ["t2", "t1"])
+        cmds.doit_ignore(TESTDB, tasks, output, ["t2", "t1"])
         got = output.getvalue().split("\n")[:-1]
         assert ["ignoring t2", "ignoring t1"] == got
         dep = Dependency(TESTDB)
@@ -261,7 +290,7 @@ class TestCmdIgnore(object):
 
     def testIgnoreGroup(self, tasks):
         output = StringIO.StringIO()
-        doit_ignore(TESTDB, tasks, output, ["g2"])
+        cmds.doit_ignore(TESTDB, tasks, output, ["g2"])
         got = output.getvalue().split("\n")[:-1]
 
         dep = Dependency(TESTDB)
@@ -275,13 +304,157 @@ class TestCmdIgnore(object):
     # if task dependency not from a group dont ignore it
     def testDontIgnoreTaskDependency(self, tasks):
         output = StringIO.StringIO()
-        doit_ignore(TESTDB, tasks, output, ["t3"])
+        cmds.doit_ignore(TESTDB, tasks, output, ["t3"])
         dep = Dependency(TESTDB)
         assert '1' == dep._get("t3", "ignore:")
         assert None == dep._get("t1", "ignore:")
 
     def testIgnoreInvalid(self, tasks):
         output = StringIO.StringIO()
-        py.test.raises(InvalidCommand, doit_ignore,
+        py.test.raises(InvalidCommand, cmds.doit_ignore,
                        TESTDB, tasks, output, ["XXX"])
 
+
+
+def pytest_funcarg__cwd(request):
+    """set cwd to parent folder of this file."""
+    def set_cwd():
+        cwd = {}
+        cwd['preivous'] = os.getcwd()
+        cwd['current'] = os.path.abspath(os.path.dirname(__file__))
+        os.chdir(cwd['current'])
+        return cwd
+    def restore_cwd(cwd):
+        os.chdir(cwd['preivous'])
+    return request.cached_setup(
+        setup=set_cwd,
+        teardown=restore_cwd,
+        scope="function")
+
+
+class TestFileWatcher(object):
+    def testInit(self, cwd):
+        file1, file2 = 'data/w1.txt', 'data/w2.txt'
+        fw = cmds.FileModifyWatcher((file1, file2))
+        # file_list contains absolute paths
+        assert os.path.abspath(file1) in fw.file_list
+        assert os.path.abspath(file2) in fw.file_list
+        # watch_dirs
+        assert os.path.join(cwd['current'], 'data') in fw.watch_dirs
+
+    def testHandleEventNotSubclassed(self):
+        fw = cmds.FileModifyWatcher([])
+        py.test.raises(NotImplementedError, fw.handle_event, None)
+
+    def testLoop(self, cwd):
+        file1, file2, file3 = 'data/w1.txt', 'data/w2.txt', 'data/w3.txt'
+        stop_file = 'data/stop'
+        fw = cmds.FileModifyWatcher((file1, file2, stop_file))
+        events = []
+        should_stop = []
+        started = []
+        def handle_event(event):
+            events.append(event.pathname)
+            if event.pathname.endswith("stop"):
+                should_stop.append(True)
+        fw.handle_event = handle_event
+
+        def loop_callback(notifier):
+            started.append(True)
+            # force loop to stop
+            if should_stop:
+                raise KeyboardInterrupt
+        loop_thread = threading.Thread(target=fw.loop, args=(loop_callback,))
+        loop_thread.daemon = True
+        loop_thread.start()
+
+        # wait watcher to be ready
+        while not started:
+            assert loop_thread.isAlive()
+
+        # write in watched file
+        fd = open(file1, 'w')
+        fd.write("hi")
+        fd.close()
+        # write in non-watched file
+        fd = open(file3, 'w')
+        fd.write("hi")
+        fd.close()
+        # write in another watched file
+        fd = open(file2, 'w')
+        fd.write("hi")
+        fd.close()
+
+        # tricky to stop watching
+        fd = open(stop_file, 'w')
+        fd.write("hi")
+        fd.close()
+        loop_thread.join(1)
+        assert not loop_thread.isAlive()
+
+        assert os.path.abspath(file1) == events[0]
+        assert os.path.abspath(file2) == events[1]
+
+
+class TestCmdAuto(object):
+    def test(self, cwd, capsys):
+        file1, file2, file3 = 'data/w1.txt', 'data/w2.txt', 'data/w3.txt'
+        stop_file = 'data/stop'
+        should_stop = []
+        started = []
+        # attach file watcher "stop" loop stuff
+        def _handle(self, event):
+            if event.pathname in self.file_list:
+                self.handle_event(event)
+                if event.pathname.endswith("stop"):
+                    should_stop.append(True)
+        cmds.FileModifyWatcher._handle = _handle
+        # stop file watcher
+        def loop_callback(notifier):
+            started.append(True)
+            if should_stop:
+                raise KeyboardInterrupt
+
+        remove_testdb()
+        # create files
+        for fx in (file1, file2, file2, stop_file):
+            fd = open(fx, 'w')
+            fd.write("hi")
+            fd.close()
+        #
+        task_list = [Task("t1", [""], [file1]),
+                     Task("t2", [""], [file2]),
+                     Task("stop", [""],  [stop_file]),]
+        run_args = (TESTDB, task_list, ["t1", "t2", "stop"], loop_callback)
+        loop_thread = threading.Thread(target=cmds.doit_auto, args=run_args)
+        loop_thread.daemon = True
+        loop_thread.start()
+
+        # wait watcher to be ready
+        while not started: assert loop_thread.isAlive()
+
+        # write in watched file ====expected=====> .  t1
+        fd = open(file1, 'w')
+        fd.write("hi")
+        fd.close()
+        # write in non-watched file ============> None
+        fd = open(file3, 'w')
+        fd.write("hi")
+        fd.close()
+        # write in another watched file ========> .  t2
+        fd = open(file2, 'w')
+        fd.write("hi")
+        fd.close()
+
+        # tricky to stop watching
+        fd = open(stop_file, 'w')
+        fd.write("hi")
+        fd.close()
+        loop_thread.join(1)
+        assert not loop_thread.isAlive()
+
+        out, err = capsys.readouterr()
+        out_lines = out.splitlines()
+        assert '.  t1' == out_lines[0]
+        assert '.  t2' == out_lines[1]
+
